<?php

declare(strict_types=1);

namespace Cryptonix_Payment\HttpClient;

class Cryptonix_Api implements Http_Client_Interface
{
    /**
     * @var string Cryptonix API URL.
     */
    private string $api_url = 'https://cryptonix.com/';

    /**
     * @var string Cryptonix API public key.
     */
    private string $public_key;

    /**
     * @var string Cryptonix API private key.
     */
    private string $private_key;

    /**
     * @var int Cryptonix API timeout in seconds.
     */
    private int $timeout = 15;

    public function __construct(string $public_key, string $private_key) {
        $this->public_key = $public_key;
        $this->private_key = $private_key;
    }

    public function send_request(string $uri, array $data = [], string $method = 'GET'): array {
        $url = rtrim($this->api_url, '/') . '/' . ltrim($uri, '/');

        $args = [
            'headers' => [
                'api-key' => $this->public_key,
                'signature' => $this->calculate_signature($data),
                'Content-Type' => 'application/json',
            ],
            'body' => wp_json_encode($data),
            'timeout' => $this->timeout,
        ];

        if (strtoupper($method) === 'POST') {
            $response = wp_remote_post($url, $args);
        } else {
            $response = wp_remote_get($url, $args);
        }

        if (is_wp_error($response)) {
            return [
                'success' => false,
                'error' => $response->get_error_message(),
            ];
        }

        if (wp_remote_retrieve_response_code( $response ) !== 200) {
            return [
                'success' => false,
                'error' => __('Invalid HTTP response code', 'cryptonix-for-woocommerce'),
                'code' => wp_remote_retrieve_response_code( $response ),
                'response' => wp_remote_retrieve_body( $response ),
            ];
        }

        return [
            'success' => true,
            'data' => json_decode(wp_remote_retrieve_body($response), true),
        ];
    }

    private function calculate_signature(array $payload): string {
        return base64_encode(hash_hmac(
            'sha512',
            wp_json_encode($payload),
            $this->private_key,
            true,
        ));
    }
}
