<?php

declare(strict_types=1);

namespace Cryptonix_Payment\Woocommerce\Gateway;

use Cryptonix_Payment\DTO\Create_Order_DTO;
use Cryptonix_Payment\Service\Order_Service;
use JetBrains\PhpStorm\NoReturn;
use WC_Payment_Gateway;

class Cryptonix_Gateway extends WC_Payment_Gateway {
    private bool $debug;

    public function __construct() {
        $this->id = 'cryptonix';
        $this->icon = '';
        $this->has_fields = true;
        $this->method_title = __('Cryptonix', 'cryptonix-for-woocommerce');
        $this->method_description = __('Cryptonix Payment.', 'cryptonix-for-woocommerce');

        $this->init_form_fields();
        $this->init_settings();

        $this->title = $this->get_option('title');
        $this->description = $this->get_option('description');
        $this->order_button_text  = __( 'Proceed to Cryptonix', 'cryptonix-for-woocommerce' );
        $this->debug = 'yes' === $this->get_option( 'debug', 'no' );

        add_action('woocommerce_update_options_payment_gateways_' . $this->id, [$this, 'process_admin_options']);

        add_action('woocommerce_thankyou_' . $this->id, [$this, 'payment_status_page']);

        add_action('woocommerce_api_cryptonix_gateway', [$this, 'handle_webhook']);
    }

    public function init_form_fields(): void {
        $this->form_fields = [
            'enabled' => [
                'title' => __( 'Enable/Disable', 'cryptonix-for-woocommerce' ),
                'type' => 'checkbox',
                'label' => __('Enable Cryptonix', 'cryptonix-for-woocommerce'),
                'default' => 'yes',
            ],
            'title' => [
                'title' => __( 'Title', 'cryptonix-for-woocommerce' ),
                'type' => 'text',
                'description' => __( 'This controls the title which the user sees during checkout.', 'cryptonix-for-woocommerce' ),
                'default' => __('Pay with crypto', 'cryptonix-for-woocommerce'),
                'desc_tip' => true,
            ],
            'description' => [
                'title' => 'Description',
                'type' => 'textarea',
                'description' => __( 'This controls the description which the user sees during checkout.', 'cryptonix-for-woocommerce' ),
                'default' => __( 'Pay with cryptocurrencies.', 'cryptonix-for-woocommerce' ),
            ],
            'public_key' => [
                'title' => __( 'Public API Key', 'cryptonix-for-woocommerce' ),
                'type' => 'text',
                'default' => '',
                'description' => __('You can manage your API keys within the Cryptonix Project Settings pages', 'cryptonix-for-woocommerce'),
            ],
            'private_key' => [
                'title' => __( 'Private API Key', 'cryptonix-for-woocommerce' ),
                'type' => 'text',
                'default' => '',
                'description' => __('You can manage your API keys within the Cryptonix Project Settings pages', 'cryptonix-for-woocommerce'),
            ],
            'webhook_secret_key' => [
                'title' => __( 'Webhook Secret Key', 'cryptonix-for-woocommerce' ),
                'type' => 'text',
                'default' => '',
                'description' => __(
                    'Using webhooks allows Cryptonix to send payment confirmation messages to the website. You can fill webhook secret key in Cryptonix Project Settings pages (Callback key)',
                    'cryptonix-for-woocommerce'
                ),
            ],
            'webhook_url' => [
                'title' => __( 'Webhook URL', 'cryptonix-for-woocommerce' ),
                'type' => 'text',
                'default' => home_url('/?wc-api=cryptonix_gateway'),
                'description' => __(
                    'This is the URL that will receive the webhooks from Cryptonix. You can use this URL in the Cryptonix Project Settings pages (Callback URL)',
                    'cryptonix-for-woocommerce'
                ),
                'custom_attributes' => [
                    'readonly' => 'readonly',
                ],
            ],
            'debug' => [
                'title' => __( 'Debug log', 'cryptonix-for-woocommerce' ),
                'type' => 'checkbox',
                'label' => __( 'Enable logging', 'cryptonix-for-woocommerce' ),
                'default' => 'no',
                'description' => __( 'When enable, Payment error log will be saved in woocommerce > Status > log', 'cryptonix-for-woocommerce' ),
            ],
        ];
    }

    public function payment_fields(): void {
        if (!$this->get_option('public_key') || !$this->get_option('private_key')) {
            echo '<div class="woocommerce-error">'. esc_html( __('Cryptonix API keys are required.', 'cryptonix-for-woocommerce') ) . '</div>';
        } else {
            echo '<p>' . esc_html( __('Payment with Cryptonix.', 'cryptonix-for-woocommerce') ) . '</p>';
        }
    }

    public function is_available(): bool {
        return 'yes' === $this->get_option('enabled');
    }

    private function log( $message, string $level = 'info', array $context = [] ) {
        if ($this->debug) {
            wc_get_logger()->log($level, $message, $context);
        }
    }

    public function process_payment( $order_id ) {
        /** @var \WC_Order $order */
        $order = wc_get_order($order_id);

        $this->log('Processing cryptonix payment for order ID '. $order_id);

        $public_key = $this->get_option('public_key');
        $private_key = $this->get_option('private_key');
        $service = new Order_Service($public_key, $private_key);

        $billing = $order->data['billing'];
        $customer_data = [
            'first_name' => $billing['first_name'],
            'last_name' => $billing['last_name'],
            'email' => $billing['email'],
            'residence' => $billing['country'],
            'address' => $billing['address_1'] . ' ' . $billing['address_2'],
        ];

        $order_dto = new Create_Order_DTO(
            $order->get_order_number(),
            $order->get_currency(),
            (float) $order->get_total()
        );
        $order_dto
            ->set_order_description( $this->get_order_description( $order ) )
            ->set_success_url( $this->get_successful_payment_url($order) )
            ->set_fail_url( $this->get_failure_payment_url($order) )
            ->set_cancel_url( $order->get_cancel_order_url() )
            ->set_client_id( (string) $order->get_customer_id() ) // $order->get_user_id()
            ->set_language( get_bloginfo('language') )
            ->set_customer_data( $customer_data );

        try {
            $response = $service->create_order($order_dto);

            if ($response['success']) {
                $order->update_status('on-hold', __('Awaiting cryptonix payment', 'cryptonix-for-woocommerce'));

                $this->log('Redirecting to Cryptonix checkout for order ID '. $order_id, 'info', [
                    'response' => $response,
                    'order_id' => $order_id,
                    'order_data' => $order_dto->to_array(),
                ]);
                $order->add_order_note(sprintf(
                    __('Order %d was created in cryptonix side. UUID: %s', 'cryptonix-for-woocommerce'),
                    $order_id, $response['data']['order']['order_uuid'] ?? '',
                ));
                return [
                    'result' => 'success',
                    'redirect' => $response['data']['checkout_url'],
                ];
            } else {
                /* translators: Show error message when we can't create an order on Cryptonix side */
                $message = \sprintf(__('Error payment order with Cryptonix: %s', 'cryptonix-for-woocommerce'), esc_html($response['error']));
                if (!empty($response['response'])) {
                    /* translators: Show specific error message if API response has it in a response */
                    \sprintf(__('Error payment order with Cryptonix: %s', 'cryptonix-for-woocommerce'), esc_html((string) (json_decode($response['response'], true)['message'] ?? $message)));
                }
                wc_add_notice($message, 'error');

                $this->log('Error creating order with Cryptonix: '. $response['error'] ?? '', 'error', [
                    'response' => $response,
                    'order_id' => $order_id,
                    'order_data' => $order_dto->to_array(),
                ]);

                return [
                    'result' => 'fail',
                ];
            }
        } catch (\Exception $e) {
            wc_add_notice(__('Error: ', 'cryptonix-for-woocommerce') . $e->getMessage(), 'error');
            $this->log('Error creating order with Cryptonix: '. $e->getMessage(), 'error', [
                'exception' => $e->__toString(),
                'order_id' => $order_id,
                'order_data' => $order_dto->to_array(),
            ]);
            return [
                'result' => 'fail',
            ];
        }
    }

    #[NoReturn]
    public function handle_webhook(): void {
        $payload = file_get_contents('php://input');
        $data = json_decode($payload, true);
        $order_data = $data['data'] ?? [];

        $this->log('Webhook received', 'info', ['payload' => $data]);

        if (empty($order_data['order_id']) || empty($data['event'])) {
            $this->log('Invalid webhook payload', 'error', ['payload' => $data]);
            wp_die('Invalid webhook payload', 'Invalid', ['response' => 400]);
        }

        if ('status_changed' !== $data['event']) {
            $this->log('Invalid webhook event', 'error', ['payload' => $data]);
            wp_die('Invalid webhook event', 'Invalid', ['response' => 400]);
        }

        $headers = getallheaders();
        $signature = $headers['Signature'] ?? '';

        if (!$this->verify_webhook_signature($payload, $signature)) {
            $this->log('Invalid webhook signature', 'error', ['payload' => $data]);
            wp_die('Invalid webhook signature', 'Unauthorized', ['response' => 401]);
        }

        $order_id = intval($order_data['external_order_id']);
        $order = wc_get_order($order_id);

        if (!$order || !$order->get_id()) {
            $this->log('Order not found for ID ' . $order_id, 'error');
            wp_die('Order not found', 'Not Found', ['response' => 404]);
        }

        switch ($order_data['status']) {
            case 'paid':
            case 'overpaid':
            case 'overdue_and_paid':
                $order->update_status('processing', __('Payment received via webhook.', 'cryptonix-for-woocommerce'));
                $this->log('Webhook confirmed payment for order ID ' . $order_id, 'info', ['payload' => $data]);
                break;

            case 'overdue':
            case 'returned':
                $order->update_status('failed', __('Payment failed via webhook.', 'cryptonix-for-woocommerce'));
                $this->log('Webhook reported failed payment for order ID ' . $order_id, 'error', ['payload' => $data]);
                break;

            default:
                $this->log('Unknown event received via webhook for order ID ' . $order_id, 'warning', ['payload' => $data]);
                break;
        }

        wp_die('Webhook processed', 'OK', ['response' => 200]);
    }

    private function verify_webhook_signature(string $payload, string $signature): bool {
        $secret_key = $this->get_option('webhook_secret_key');

        if (!$secret_key) {
            $this->log('Webhook secret key is missing', 'error');
            return false;
        }

        $expected_signature = hash_hmac(
            'sha512',
            $payload,
            $secret_key,
            true
        );

        return hash_equals($expected_signature, base64_decode($signature));
    }

    private function get_successful_payment_url( \WC_Order $order ): string {
        return $this->get_return_url($order) . '&payment=success&order_id=' . $order->get_id();
    }

    private function get_failure_payment_url( \WC_Order $order ): string {
        return $this->get_return_url($order). '&payment=failed&order_id='. $order->get_id();
    }

    private function get_order_description( \WC_Order $order ): ?string {
        try {
            $order_items = array_map( function( $item ) {
                return $item['quantity'] . ' x ' . $item['name'];
            }, $order->get_items() );

            $description = mb_substr( implode( ', ', $order_items ), 0, 200 );
        } catch ( \Exception $e ) {
            $description = null;
        }

        return $description;
    }

    public function payment_status_page(): void {
        $order_key = wc_clean( wp_unslash( $_GET['key'] ?? '' ) );
        $order_id = wc_get_order_id_by_order_key($order_key);
        $order = wc_get_order($order_id);

        if (isset($_GET['payment'])) {
            $payment_status = wc_clean( wp_unslash( $_GET['payment'] ) );

            if ($payment_status === 'success') {
                echo '<h2>' . esc_html( __('Thank you for your order!', 'cryptonix-for-woocommerce') ) . '</h2>';
                echo '<p>' . esc_html( __('Your payment has been successfully processed.', 'cryptonix-for-woocommerce') ) . '</p>';
                echo '<p>' . esc_html( __('Order ID: ', 'cryptonix-for-woocommerce') ) . esc_html($order->get_order_number()) . '</p>';
            } elseif ($payment_status === 'failed') {
                echo '<h2>' . esc_html( __('Payment Failed', 'cryptonix-for-woocommerce') ) . '</h2>';
                echo '<p>' . esc_html( __('Unfortunately, your payment could not be processed. Please try again.', 'cryptonix-for-woocommerce') ) . '</p>';
                echo '<p>' . esc_html( __('Order ID: ', 'cryptonix-for-woocommerce') ) . esc_html($order->get_order_number()) . '</p>';
            }
        } else {
            $this->log('Invalid payment status for order ID '. $order_id, 'error', [
                'order_id' => $order_id,
                'payment_status' => wc_clean( wp_unslash( $_GET['payment'] ?? '' ) ),
            ]);
            echo '<h2>' . esc_html( __('Invalid payment status', 'cryptonix-for-woocommerce') ) . '</h2>';
        }
    }
}
